import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

/**
 * Counter that displays a level and score with text prefixes.
 * 
 * @author Michael Kolling (amended by David J. Barnes)
 * @version 1.0.2
 */
public class Counter extends Actor
{
    private static final Color textColor = Color.BLACK;
    private static final int FONT_SIZE = 16;
    private static final int IMAGE_HEIGHT = 20;
    private static final String LEVEL_TEXT = "Level: ";
    private static final String SCORE_TEXT = "Score: ";
    
    // The game level.
    private int level = 0;
    // The game score.
    private int score = 0;
    private String text;

    public Counter()
    {
        int stringLength = (LEVEL_TEXT.length() + SCORE_TEXT.length() + 3) * FONT_SIZE;

        setImage(new GreenfootImage(stringLength, IMAGE_HEIGHT));
        GreenfootImage image = getImage();
        image.setColor(textColor);
        Font font = image.getFont();
        image.setFont(font.deriveFont((float) FONT_SIZE));  // use larger font
        updateImage();
    }
    
    public void act()
    {
    }

    /**
     * Add points to the score.
     * @param points The points to be added.
     */
    public void add(int points)
    {
        this.score += points;
        updateImage();
    }

    /**
     * Subtract points from the score.
     * @param points The points to be subtracted.
     */
    public void subtract(int points)
    {
        this.score -= points;
        updateImage();
    }

    /**
     * Return the score.
     * @return The score.
     */
    public int getScore()
    {
        return score;
    }
    
    /**
     * Set the current level.
     * @param level The current level.
     */
    public void setLevel(int level)
    {
        this.level = level;
        updateImage();
    }

    /**
     * Update the image to include the current level and score.
     */
    private void updateImage()
    {
        String text = LEVEL_TEXT + level + " " + SCORE_TEXT + score;
        int stringLength = text.length() * FONT_SIZE;
        GreenfootImage image = getImage();
        if(image.getWidth() < stringLength) {
            image.scale(stringLength, IMAGE_HEIGHT);
        }
        image.clear();
        image.drawString(text, 1, 12);
    }
}
