import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * Elephants trample beetles, but they get distracted by
 * chips, which are not good for them.
 * 
 * @author David J. Barnes
 * @version 2008.08.01
 */
public class Elephant extends Actor
{
    /**
     * Act - do whatever the Robot wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        ElephantWorld world = (ElephantWorld) getWorld();
        List<Actor> beetles = world.getObjects(Beetle.class);
        if(beetles.size() > 0) {
            // Head towards the first one.
            Beetle beetle = (Beetle) beetles.get(0);
            // Check to see if the beetle is already underfoot.
            // This may be the case if it has teleported.
            if(isTrampling(beetle)) {
                squash(beetle);
            }
            else {
                moveTowards(beetle);
                if(isTrampling(beetle)) {
                    squash(beetle);
                }
                else if(foundChips()) {
                    world.incrementScore(1);
                    world.removeObject(this);
                }
                else {
                    // See if we have crashed into another elephant.
                    checkForCollision();
                }
            }
        }
    }
    
    /**
     * Move towards the given actor.
     * Movement is in all eight directions.
     * @param actor The actor to be moved towards.
     */
    private void moveTowards(Actor actor)
    {
        int x = getX(), y = getY();
        int targetX = actor.getX(), targetY = actor.getY();
        if(targetX > x) {
            x++;
        }
        else if(targetX < x) {
            x--;
        }
        if(targetY > y) {
            y++;
        }
        if(targetY < y) {
            y--;
        }
        setLocation(x,y);
    }
    
    /**
     * See if we have collided into other elephants.
     * If so, all get eliminated.
     */
   private void checkForCollision()
    {
        int x = getX(), y = getY();
        ElephantWorld world = (ElephantWorld) getWorld();
        List<Actor> elephants = world.getObjectsAt(x, y, Elephant.class);
        if(elephants.size() > 1) {
            // We have collided. :(
            world.incrementScore(elephants.size());
            world.removeObjects(elephants);
        }
    }
    
    /**
     * Check whether we have trampled the beetle.
     * @param beetle The beetle to be trampled.
     * @return true If the beetle is being trampled, false otherwise.
     */
    private boolean isTrampling(Beetle beetle)
    {
        return getX() == beetle.getX() && getY() == beetle.getY();
    }
    
    /**
     * We have squashed the beetle. Remove it from the game.
     * @param beetle The beetle that has been squashed.
     */
    private void squash(Beetle beetle)
    {
        Greenfoot.playSound("sounds/splat.wav");
        getWorld().removeObject(beetle);
    }
    
    /**
     * Check whether there are chips at this location.
     * @return true if there are chips, false otherwise.
     */
    private boolean foundChips()
    {
        List<Actor> chips = getWorld().getObjectsAt(getX(), getY(), Chips.class);
        return chips.size() > 0;
    }
}
