import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A world in which rampaging elephants trample a poor
 * defenceless beetle. Only their desire for chips and
 * a tendency to crash into each other can save our hero.
 * 
 * @author David J. Barnes
 * @version 2008.08.01
 */
public class ElephantWorld extends World
{
    private int level;
    // A multiplier to determine how many elephants to create.
    private static final int ELEPHANT_LEVEL_MULTIPLIER = 4;
    // A multiplier to determine how many chips to create.
    private static final int CHIPS_LEVEL_MULTIPLIER = 2;
    // The probablility of there being chips.
    private static final int PROBABILITY_OF_CHIPS = 75;
    // The current score.
    private Counter score;

    
    /**
     * Constructor for objects of class RobotWorld.
     * 
     */
    public ElephantWorld()
    {    
        super(35, 35, 20);
                
        setPaintOrder(Elephant.class, Beetle.class, Counter.class);
        setActOrder(Beetle.class, Elephant.class);
        level = 0;
        score = new Counter();
        
        startNextLevel();
    }
    
    /**
     * Set up the population of one beetle and multiple
     * elephants and chips for the next level.
     */
    public void startNextLevel()
    {
        int width = getWidth(), height = getHeight();
        
        level++;
        addObject(score, 7, height - 3);
        score.setLevel(level);
        
        // Create the elephants.
        for(int i = 0; i < ELEPHANT_LEVEL_MULTIPLIER * level; i++) {
            addObject(new Elephant(), Greenfoot.getRandomNumber(width), Greenfoot.getRandomNumber(height));
        }
        
        // Create the chips.
        for(int i = 0; i < CHIPS_LEVEL_MULTIPLIER * level; i++) {
            if(Greenfoot.getRandomNumber(100) < PROBABILITY_OF_CHIPS) {
                addObject(new Chips(), Greenfoot.getRandomNumber(width), Greenfoot.getRandomNumber(height));
            }
        }
        
        // Create the beetle.
        addObject(new Beetle(), Greenfoot.getRandomNumber(width), Greenfoot.getRandomNumber(height));        
    }
    
    /**
     * Check for the end of a level or the game as a whole.
     */
    public void act()
    {
        if(getObjects(Beetle.class).size() == 0) {
            // Game over.
            Greenfoot.stopSimulation();
        }
        else if(getObjects(Elephant.class).size() == 0) {
            // Level over.
            removeObjects(getObjects(Actor.class));
            startNextLevel();
        }
        else {
            // The game goes on ...
        }
    }
    
    /**
     * Increment the score by the given amount.
     * @param amount The score increment.
     */
    public void incrementScore(int amount)
    {
        score.add(amount);
    }
            
}
