import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Functionality to deliver a movement request from the keyboard.
 * 
 * @author David J. Barnes
 * @version 2008.08.01
 */
public class MoveableActor extends Actor
{
            
    /**
     * Return a single character representing a movement request.
     * @return The movement request.
     */
    public char getMovementChar()
    {
        String movement = getMovementRequest();
        
        // Return single characters as they are.
        if(movement.length() == 1) {
            return movement.charAt(0);
        }
        
        // Map more complex strings to movement commands.
        if(isMoveLeft(movement)) {
            return Movements.LEFT;
        }
        else if(isMoveRight(movement)) {
            return Movements.RIGHT;
        }
        else if(isMoveUp(movement)) {
            return Movements.UP;
        }
        else if(isMoveDown(movement)) {
            return Movements.DOWN;
        }
        else {
            return Movements.DO_NOTHING;
        }
    }
        
    /**
     * Blockingly read a movement request from the keyboard.
     * @return The request.
     */
    public String getMovementRequest()
    {
        String key = Greenfoot.getKey();
        
        while(key == null) {
            Greenfoot.delay(1);
            key = Greenfoot.getKey();
        }
        return key;
    }
    
    /**
     * Determine whether movement is a move to the left.
     * @param movement The movement request.
     * @return true If the movement is a request to move left, false otherwise.
     */
    private boolean isMoveLeft(String movement)
    {
        return movement.equals("left") || 
               movement.equals("" + Movements.LEFT);
    }
    
    /**
     * Determine whether movement is a move to the right.
     * @param movement The movement request.
     * @return true If the movement is a request to move right, false otherwise.
     */
    private boolean isMoveRight(String movement)
    {
        return movement.equals("right") ||
               movement.equals("" + Movements.RIGHT);
    }
    
    /**
     * Determine whether movement is a move up.
     * @param movement The movement request.
     * @return true If the movement is a request to move up, false otherwise.
     */
    private boolean isMoveUp(String movement)
    {
        return movement.equals("up") ||
               movement.equals("" + Movements.UP);
    }
    
    /**
     * Determine whether movement is a move down.
     * @param movement The movement request.
     * @return true If the movement is a request to move down, false otherwise.
     */
    private boolean isMoveDown(String movement)
    {
        return movement.equals("down") ||
               movement.equals("" + Movements.DOWN);
    }
}
